#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Jul 31 18:21:01 2020

@author: andremounier
"""

# In[1] Importation des modules

import numpy as np
from scipy.integrate import quad, simps


# In[2] Importation des constantes

#Constantes physiques
fichier_CSTE = open("Constantes.txt","r").read().split('\n\n')
Constantes = {}
for elem in fichier_CSTE:
	entrée = elem.split(',')
	Constantes[entrée[0]] = [float(entrée[i]) for i in range(1,len(entrée))]

#Données atmosphériques
fichier_ATM = open("Données_Atmosphère.txt","r").read().split('\n\n')
Données_Atmosphère = {}
for elem in fichier_ATM:
	entrée = elem.split(',')
	Données_Atmosphère[entrée[0]] = [float(entrée[i]) for i in range(1,len(entrée))]

#Données liées au CO2
fichier_CO2 = open("Données_CO2.txt","r").read().split('\n\n')
Données_CO2 = {}
for elem in fichier_CO2:
	entrée = elem.split(',')
	Données_CO2[entrée[0]] = [float(entrée[i]) for i in range(1,len(entrée))]

#Données liées à la vapeur d'eau
fichier_H2O = open("Données_H2O.txt","r").read().split('\n\n')
Données_H2O = {}
for elem in fichier_H2O:
	entrée = elem.split(',')
	Données_H2O[entrée[0]] = [float(entrée[i]) for i in range(1,len(entrée))]


# In[3] Définition des constantes

h			= Constantes['Constante de Planck'][0]
c			= Constantes['Vitesse de la lumière dans le vide'][0]
kB			= Constantes['Constante de Boltzmann'][0]
P_standard 	= Constantes['Pression standard'][0]
T_standard 	= Constantes['Température standard'][0]
Na			= Constantes["Constante d'Avogadro"][0]
R			= Constantes['Constante des gaz pafaits'][0]
sigma_SB		= Constantes['Constante de Stefan-Boltzmann'][0]

Coeffs_température 	= Données_Atmosphère['Évolution de la température']
Z_T					= Données_Atmosphère['Altitude de la tropopause'][0]
h_r					= Données_Atmosphère['Hauteur caractéristique en pression'][0]
dTdz_REF				= Données_Atmosphère['Gradient thermique de référence'][0]

nu_coupure_CO2		= Données_CO2["Nombres d'onde à coupure"]
pente_AT_CO2			= Données_CO2['Pentes de la section efficace']
intersection_AT_CO2 	= Données_CO2["Ordonnées à l'origine de la section efficace"]

h_e_REF 			= Données_H2O["Hauteur caractéristique de la vapeur d'eau"][0]
x0_H2O_REF		= Données_H2O["Fraction molaire standard en vapeur d'eau"][0]
coupure			= Données_H2O["Nombre d'onde de coupure"][0]
Coeffs_avant 	= Données_H2O['Coefficients avant coupure']
Coeffs_après 	= Données_H2O['Coefficients après coupure']
Coeffs_CYES 		= Données_H2O['Coefficients du Continuum']


# In[4] Fonctions préliminaires

def Température_REF(z):
	"""
	Température selon l'altitude
	
	 - Entrée(s) : Altitude (en km)
	 - Sortie(s) : Température (en K)
	"""
	res = 0
	for i in range(len(Coeffs_température)):
		res+= Coeffs_température[i]*z**i
	return res

def Température_ajustée(z, Tas = 294, dTdz = dTdz_REF):
	"""
	Température selon l'altitude
	
	 - Entrée(s) : Altitude (en km), Température en bas de l'atmoshère (en K), Gradient thermique (en K/km)
	 - Sortie(s) : Température (en K)
	"""
	return Température_REF(z)+(Tas-294)+(dTdz-dTdz_REF)*z

def B(nu,T):
    """
	Loi de Planck
	
	 - Entrée(s) : Nombre d'onde (en cm-1) et Température
	 - Sortie(s) : Spectre de luminance (en W/m2.cm-1)
	"""
    nu = 1e2*nu #passage en m-1
    return 1e2*2*np.pi*h*nu**3*c**2/(np.exp(c*h*nu/kB/T)-1)

def Stefan_Boltzmann(T):
	"""
	Loi de Stefan-Boltzmann
		
	 - Entrée(s) : Température
	 - Sortie(s) : Luminance (en W/m2)
	"""
	return sigma_SB*T**4


# In[5] Fonctions de modélisation des sections efficaces

def sigma_CO2(nu):
	"""
	Calcul de la section efficace du CO2, après tri 
	
	 - Entrée(s) : Nombre d'onde (en cm-1)
	 - Sortie(s) : Section efficace (en m2)
	"""
	compteur = 0
	while nu > nu_coupure_CO2[compteur+1]:
		compteur+=1
	return np.exp(pente_AT_CO2[compteur]*nu+intersection_AT_CO2[compteur])

def sigma_H2O(nu):
	"""
	Calcul de la section efficace de la vapeur d'eau
	
	 - Entrée(s) : Nombre d'onde (en cm-1)
	 - Sortie(s) : Section efficace (en m2)
	"""
	res = 0
	if nu < coupure:
		for i in range(len(Coeffs_avant)):
			res+=Coeffs_avant[i]*nu**i
		return np.exp(res)
	else : 
		for i in range(len(Coeffs_après)):
			res+=Coeffs_après[i]*nu**i
	return np.exp(res)

def Continuum(nu):
	"""
	Prise en compte du continuum
	
	 - Entrée(s) : Nombre d'onde (en cm-1)
	 - Sortie(s) : Section efficace (en m2/fact_H2O)
	"""
	res = 0
	for i in range(len(Coeffs_CYES)):
		res+=Coeffs_CYES[i]*nu**i
	return np.exp(res)


# In[6] Fonctions de calcul des spectres de luminance

def F_surface_up_LW(Ts):
	"""
	Calcul de la luminance du rayonnement émis par la surface
	
	 - Entrée(s) : Température de surface (en K)
	 - Sortie(s) : Luminance (en W/m2)
	"""
	return Stefan_Boltzmann(Ts)


def F_atmosphere_up_LW(X_CO2, fact_H2O, Ts = 294, Tas = 294, dTdz = dTdz_REF, h_e = h_e_REF, Plage_nu = np.linspace(1,2500,50), CYES = True):
	"""
	Calcul de la luminance et du spectre du rayonnement sortant de l'atmopshère vers l'espace
	
	 - Entrée(s) : CO2 (en ppmv), Facteur de vapeur d'eau, Température de surface (en K), Température de bas d'atmosphère (en K), Gradient thermique (en K/km), Hauteur caractéristique de décroissance de la vapeur d'eau (en km), Plage de nombre d'onde (en cm-1), Continuum
	 - Sortie(s) : Luminance (en W/m2), Spectre d'émission total (en W/m2.cm-1), Spectre d'émission de l'atmosphère (en W/m2.cm-1)
	"""
	
	h_c = h_r/2
	h_x = h_r*h_e/(h_r+2*h_e)
	
	n0_CO2 = X_CO2*1e-6*P_standard*Na/R/T_standard
	n0_H2O = x0_H2O_REF*fact_H2O*P_standard/R/T_standard*Na
	
	I_surf = np.zeros(len(Plage_nu))
	I_atm  = np.zeros(len(Plage_nu))
    
	for i,nu in enumerate(Plage_nu):
		N_CO2 = n0_CO2*h_c*1e3*sigma_CO2(nu)
		if CYES:
			N_H2O = n0_H2O*h_x*1e3*(sigma_H2O(nu)+Continuum(nu)*fact_H2O)
		else : 
			N_H2O = n0_H2O*h_x*1e3*sigma_H2O(nu)

		I_surf[i] = B(nu,Ts)*np.exp(-N_CO2*(1-np.exp(-Z_T/h_c))-N_H2O*(1-np.exp(-Z_T/h_x)))
		I_atm[i]  = quad(lambda z: (N_CO2/h_c*np.exp(-z/h_c)+N_H2O/h_x*np.exp(-z/h_x))*B(nu,Température_ajustée(z,Tas,dTdz))
                       *np.exp(N_CO2*(np.exp(-Z_T/h_c)-np.exp(-z/h_c)))
                       *np.exp(N_H2O*(np.exp(-Z_T/h_x)-np.exp(-z/h_x))),0,Z_T)[0]
    
	I = I_surf+I_atm
	Luminance = simps(I,Plage_nu)
    
	return Luminance, I, I_atm

def F_atmosphere_down_LW(X_CO2, fact_H2O, Tas = 294, dTdz = dTdz_REF, h_e = h_e_REF, Plage_nu = np.linspace(1,2500,50), CYES = True):
	"""
	Calcul de la luminance et du spectre d'émission
	
	 - Entrée(s) : CO2 (en ppmv), Facteur de vapeur d'eau, Température de bas d'atmosphère (en K), Gradient thermique (en K/km), Hauteur caractéristique de décroissance de la vapeur d'eau (en km),Plage de nombre d'onde (en cm-1), Continuum
	 - Sortie(s) : Luminance (en W/m2), Spectre d'émission total (en W/m2.cm-1), Spectre d'émission de l'atmosphère (en W/m2.cm-1)
	"""
	
	h_c = h_r/2
	h_x = h_r*h_e/(h_r+2*h_e)
	
	n0_CO2 = X_CO2*1e-6*P_standard*Na/R/T_standard
	n0_H2O = x0_H2O_REF*fact_H2O*P_standard/R/T_standard*Na
	
	I_atm = np.zeros(len(Plage_nu))
    
	for i,nu in enumerate(Plage_nu):
		N_CO2 = n0_CO2*h_c*1e3*sigma_CO2(nu)
		if CYES:
			N_H2O = n0_H2O*h_x*1e3*(sigma_H2O(nu)+Continuum(nu)*fact_H2O)
		else : 
			N_H2O = n0_H2O*h_x*1e3*sigma_H2O(nu)
		
		étape1 = quad(lambda z: (N_CO2/h_c*np.exp(-z/h_c)+N_H2O/h_x*np.exp(-z/h_x))*B(nu,Température_ajustée(z,Tas,dTdz))
                       *np.exp(N_CO2*(np.exp(-z/h_c)-1))
                       *np.exp(N_H2O*(np.exp(-z/h_x)-1)),0,0.5) 
        
		étape2 = quad(lambda z: (N_CO2/h_c*np.exp(-z/h_c)+N_H2O/h_x*np.exp(-z/h_x))*B(nu,Température_ajustée(z,Tas,dTdz))
                        *np.exp(N_CO2*(np.exp(-z/h_c)-1))
                        *np.exp(N_H2O*(np.exp(-z/h_x)-1)),0.5,Z_T)
        
		I_atm[i] = étape1[0] + étape2[0]
    
	I = I_atm
	Luminance = simps(I,Plage_nu)
    
	return Luminance, I, I_atm